clear all; close all;

[y, fs, nbits] = wavread('chopin_pollini.wav');

% y vettore riga contenente y(n), n=1:length(y)
% fs frequenza di campionamento in Hz
% nbits numero di bit per campione (8 o 16)

fprintf('\nPer il file wave "chopin_pollini.wav": \n');
fprintf(' frequenza di campionamento = %d Hz \n', fs);
fprintf(' numero di bit per campione = %d bit \n', nbits);
fprintf(' durata del brano = %5.2f s \n\n', length(y)/fs);

%sound(y,fs);                                % ascolto in ambiente windows/linux

fprintf(' premere un tasto per ascoltare il brano...\n');
pause;                                      % premere un tasto per proseguire
sound(y, fs);                       % ascolto dei primi 2 secondi
pause(length(y)/fs); 

fprintf(' premere un tasto per ascoltarne i primi 2 secondi...\n');
pause;                                      % premere un tasto per proseguire

sound(y(1:2*fs), fs);                       % ascolto dei primi 2 secondi
pause(2);                                   % aspetta 2 secondi (il suono va in background)

fprintf(' premere un tasto per proseguire...\n\n');
pause

%*********************************************************************
% analisi nel tempo

fprintf('ANALISI NEL TEMPO\n\n');

figure(1);
plot(y);                                    % disegna y(n), n=1:length(y) vs n
                                            % si puo` "zoomare" col bottone +
title('Andamento del segnale');
xlabel('n');

fprintf('La figura 1 visualizza l''andamento nel tempo in funzione di n:\n');
fprintf(' USARE LA LENTE (TASTO +) PER ZOOMARE SUL SEGNALE.\n');

fprintf(' premere un tasto per proseguire...\n\n');
pause                                       % premere un tasto per proseguire
close(figure(1));
figure(2)
t = linspace(1/fs, length(y)/fs, length(y));   % asse dei tempi in secondi
plot(t,y);                                     % provare anche stem(t,y,'-k.')
axis([0.1 0.3 min(y) max(y)])                  % zoom 0.1---0.3 s
xlabel('nT (s)'); ylabel('y(nT)'); title('Andamento nel tempo (0.1-0.3 s)')
grid;

fprintf('In figura 2 il tempo e` invece in secondi.\n');
fprintf(' premere un tasto per proseguire...\n\n')
pause

%**********************************************************************
% analisi in frequenza

fprintf('ANALISI IN FREQUENZA\n\n');

n = 0.04*fs;                              % analisi su 40 ms
Y = fft(y(fs:fs+(n-1)));                  % calcolo DFT su 40 ms di segnale (da 1s a 1.04s)
f = linspace(0,fs/n*(n-1),n);             % asse delle frequenze: 0---fs Hz
close(figure(2));
figure(3);
subplot(3,1,1);

plot(f/1000,abs(Y))                       % modulo
title('Modulo DFT (segnale tra 1s e 1.04s)');
xlabel('f (kHz)'); ylabel('|Y(f)|');
axis([0 fs/1000 min(abs(Y))  max(abs(Y))]);

subplot(3,1,2);
                                          % modulo in dB (e` piu` significativo)
plot(f/1000,20*log10(abs(Y)));
title('Modulo in dB (segnale tra 1s e 1.04s)');
xlabel('f (kHz)'); ylabel('|Y(f)| (dB)');
maxy = max(20*log10(abs(Y))); miny = maxy-90;
axis([0 fs/1000 miny maxy]); 

subplot(3,1,3);

plot(f/1000,angle(Y));                    % fase
title('Fase (segnale tra 1s e 1.04s)');
xlabel('f (kHz)'); ylabel('fase (Y(f)) (rad)');
axis([0 fs/1000 min(angle(Y)) max(angle(Y))]);

fprintf('In figura 3 e` raffigurata la DFT del segnale.\n');
fprintf(' premere un tasto per proseguire...\n\n');
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
close(figure(3));
N=8192;
FS=44100;
x=cos(2*pi*1000*(0:1:N-1)/44100)';
figure(2)
W=blackman(N);
W=N*W/sum(W); % scaling of window
f=((0:N/2-1)/N)*FS;

fprintf('Premi per visualizzare lo spettro del segnale e del segnale finestrato via FFT\n');
pause;

xw=x.*W;
subplot(3,2,1);plot(0:N-1,x);
axis([0 1000 -1.1 1.1]);
title('a) Cosine signal x(n)')

subplot(3,2,3);plot(0:N-1,xw);axis([0 8000 -4 4]);
title('c) Cosine signal x_w(n)=x(n) \cdot w(n) with window')

X=20*log10(abs(fft(x,N))/(N/2));
subplot(3,2,2);plot(f,X(1:N/2));
axis([0 10000 -80 10]);
ylabel('X(f)');
title('b) Spectrum of cosine signal')

Xw=20*log10(abs(fft(xw,N))/(N/2));
subplot(3,2,4);plot(f,Xw(1:N/2));
axis([0 10000 -80 10]);
ylabel('X(f)');
title('d) Spectrum with Blackman window')

s=y(1:N).*W;
subplot(3,2,5);plot(0:N-1,s);axis([0 8000 -1.1 1.1]);
xlabel('n \rightarrow');
title('e) Audio signal x_w(n) with window')

Sw=20*log10(abs(fft(s,N))/(N/2));
subplot(3,2,6);plot(f,Sw(1:N/2));
axis([0 10000 -80 10]);
ylabel('X(f)');
title('f) Spectrum with Blackman window')
xlabel('f in Hz \rightarrow');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(' premere un tasto per proseguire...\n\n');
pause;
close(figure(2));
figure(1)
[signal,FS,NBITS]=wavread('ton2');
fprintf(' premere un tasto per ascoltare ton2.wav...\n\n');
pause;
sound(signal,fs);
fprintf(' premere un tasto per visualizzare spettrogrammam e FFT...\n\n');
pause;

subplot(211);plot(signal);title('Signal x(n)');
ylabel('x(n) \rightarrow');
xlabel('n \rightarrow');
axis([0 length(signal) -1 1]);
subplot(212);
waterfspec(signal,256,256,512,FS,20,-100);

fprintf(' premere un tasto per proseguire...\n\n');
pause;

%**********************************************************
% elaborazione elementare nel tempo

fprintf('ELABORAZIONI SUL SEGNALE\n\n');

rampaoff = linspace(1,0,length(y)).';       % vettore colonna = rampa decrescente
                                            %                   da 1 a 0
yoff = y.*rampaoff;                         % moltiplica campione per campione
close(figure(1));
figure(4)
plot(t,yoff);
xlabel(' nT (s)'); ylabel('y(nT)'); title('Andamento nel tempo con rampa decrescente');

fprintf('1) Stai ascoltando il segnale moltiplicato per una rampa decrescente\n');
fprintf('   (visualizzato in figura 4).\n'); 
sound(yoff,fs);
pause(length(yoff)/fs)

fprintf('   premere un tasto per proseguire...\n\n');
pause;

fprintf('2) Stai ascoltando y(-t): cosa significa?\n');
sound(flipud(y),fs);                        % ascolto di y(-n): cosa succede a
                                            %                   modulo e fase?
pause(length(y)/fs)

fprintf('   premere un tasto per proseguire...\n\n');
pause;
close(figure(1));
fprintf('3) Stai ascoltando y(t) acquisito a fs e riprodotto ERRONEAMENTE a fs/2:\n');
%fprintf('   cosa succede alla Trasf. di Fourier?\n');
sound(y(1:length(y)/2),fs/2);               % cosa stiamo ascoltando?    
pause(length(y)/2/(fs/2))

fprintf('   premere un tasto per proseguire...\n\n');
pause;

fprintf('4) Stai ascoltando y(t) acquisito a fs, campionato a fs/8 e riprodotto a fs/8:\n');
fprintf('   vedi sul workspace le dimensioni delle variabili y (originale) e ydec_ \n');
ydec_ = y(1:8:length(y));                   % decima (senza filtrare)
sound(ydec_,fs/8);                          % cosa stiamo ascoltando?
pause(length(ydec_)/(fs/8));

fprintf('   premere un tasto per proseguire...\n\n');
pause;

fprintf('5) Stai ascoltando y(t) acquisito a fs, filtrato a fs/16, decimato per 8 e riprodotto a fs/8\n');
fprintf('   vedi sul workspace le dimensioni delle variabili y (originale) e ydec8 \n');
ydec8 = decimate(y,8);                       % filtra e decima di un fattore 8
sound(ydec8,fs/8);                           % cosa stiamo ascoltando?
pause(length(ydec8)/(fs/8));

fprintf('   premere un tasto per proseguire...\n\n');
pause;

fprintf('6) Stai ascoltando y(t) acquisito a fs, filtrato a fs/16, decimato per 16 e riprodotto a fs/16\n');
fprintf('   vedi sul workspace le dimensioni delle variabili y (originale) e ydec16 \n');
ydec16 = decimate(y,16);                       % filtra e decima di un fattore 8
sound(ydec16,fs/16);                           % cosa stiamo ascoltando?
pause(length(ydec16)/(fs/16));
fprintf('   premere un tasto per proseguire...\n\n');
pause;

% filtraggio IIR passa-basso: ordine=4
%                             banda-passante=[0, fs/80];   ripple=1 dB
%                             banda attenuata=[fP, fs/2],fP>fs/80; attenuazione 40 dB
figure(5);
[b,a] = ellip(4,1,40,1/40);            % b(z)/a(z)
[H,F] = freqz(b,a,1024,'whole',fs);    % H(F)=B(F)/A(F)
figure(5);
plot(F/1000,20*log10(abs(H)));                                             
xlabel('f (kHz)'); ylabel('|H| (dB)'); axis([0 fs/1000 -60 1]);
title('Modulo (in dB) della risposta in frequenza del filtro passa basso');

yfilt=filter(b,a,y);
fprintf('6) Stai ascoltando y(t) acquisito a fs, filtrato passa-basso a fs/80\n')
fprintf('   (con il filtro la cui risposta in frequenza e` in figura 5)\n');
%fprintf('   e riprodotto a fs: cosa succede alla Trasf. di Fourier?\n\n');
sound(yfilt,fs);                       % confronta con y
pause(length(yfilt)/fs);
